use super::freestyle::Freestyle;
use super::DeviceInfo;
use super::Glucometer;

use hidapi::HidApi;

type HidCallback = fn(&hidapi::HidApi) -> Box<dyn Glucometer>;

struct DriverLookup {
	info: DeviceInfo,
	callback: HidCallback,
}

pub fn list_hid_devices() {
	println!("Printing all available hid devices:");

	match HidApi::new() {
		Ok(api) => {
			for device in api.device_list() {
				println!("{:#?}", device);
			}
		},

		Err(e) => {
			eprintln!("Error: {}", e);
		},
	}
}

pub fn hid_matches(hid_info: &hidapi::DeviceInfo, driver_info: &DeviceInfo) -> bool {
	match driver_info {
		DeviceInfo::Hid(vendor_id, product_id) => {
			println!("Driver id {:X}:{:X}", *vendor_id, *product_id);
			println!("Device id {:X}:{:X}", hid_info.vendor_id(), hid_info.product_id());
			if hid_info.vendor_id() == *vendor_id
			    && hid_info.product_id() == *product_id {
				return true;
			}

			false
		}
	}
}

pub fn find_hid_devices() -> Option<Box<dyn Glucometer>> {
	let hidapi = hidapi::HidApi::new().unwrap();
	let mut drivers_lookup = Vec::new();

	drivers_lookup.push(DriverLookup { info: Freestyle::info(), callback: Freestyle::new });

	for device_info in hidapi.device_list() {
		for drv in &drivers_lookup {
			if hid_matches(device_info, &drv.info) {
				return Some((drv.callback)(&hidapi));
			}
		}
	}

	None
}
