use std::fs::File;
use cairo::{Context, ImageSurface, Rectangle};

use super::series::{AbstractSeries, Point, Series};
use super::axis::Axis;

pub struct DotSeries {
	series: AbstractSeries,
	icon_path: String,
	icon: ImageSurface,
	icon_size: f64,
}

impl DotSeries {
	pub fn new(id: i32, name: String, dot_path: String) -> Self {
		let mut file = File::open(&dot_path).expect("Icon not found");
		let icon = ImageSurface::create_from_png(&mut file).expect("Icon is not a PNG file");

		Self {
			series: AbstractSeries::new(id, name),
			icon_path: dot_path,
			icon: icon,
			icon_size: 32.,
		}
	}
}

impl Series for DotSeries {
	fn add_point(&mut self, point: Point) {
		self.series.add_point(point);
	}

	fn nb_points(&self) -> usize {
		self.series.nb_points()
	}

	fn get_id(&self) -> i32 {
		self.series.get_id()
	}

	fn draw_legend(&self, context: &Context, surface: &Rectangle, point: Point) {
		context.set_source_surface(&self.icon, point.x(), point.y() - self.icon_size / 2.);
		context.paint();
		context.stroke();

		context.move_to(point.x() + self.icon_size, point.y() + self.icon_size / 4.);
		context.set_source_rgb(0., 0., 0.);
		self.series.draw_text(context, &self.series.name);
		context.stroke();
	}

	fn draw(&self, context: &Context, surface: &Rectangle, x: &dyn Axis, y: &dyn Axis) {
		let mut cnt = 0;

		loop {
			if cnt + 1 >= self.nb_points() {
				break;
			}

			let current = AbstractSeries::translate_coord_scale(surface, &self.series.points[cnt], x, y);

			context.set_source_surface(&self.icon, current.x() - self.icon_size / 2., current.y() - self.icon_size / 2.);
			context.paint();

			cnt += 1;
		}

		context.stroke();
	}
}
