use cairo::{Context,  FontSlant, FontWeight, Rectangle};
use super::axis::Axis;

#[derive(Debug, Copy, Clone)]
pub struct Point {
	x: f64,
	y: f64,
}

pub trait Series {
	fn add_point(&mut self, point: Point);
	fn nb_points(&self) -> usize;
	fn get_id(&self) -> i32;
	fn draw(&self, context: &Context, surface: &Rectangle, x: &dyn Axis, y: &dyn Axis);
	fn draw_legend(&self, context: &Context, surface: &Rectangle, point: Point);
}

#[derive(Debug, Clone)]
pub struct AbstractSeries {
	pub id: i32,
	pub name: String,
	pub points: Vec<Point>,
}

impl Point {
	pub fn new(x: f64, y: f64) -> Self {
		Self {
			x: x,
			y: y,
		}
	}

	pub fn x(&self) -> f64 {
		self.x
	}

	pub fn y(&self) -> f64 {
		self.y
	}
}

impl AbstractSeries {
	pub fn new(id: i32, name: String) -> Self {
		Self {
			id: id,
			name: name,
			points: Vec::new(),
		}
	}

	pub fn add_point(&mut self, point: Point) {
		self.points.push(point);
	}

	pub fn nb_points(&self) -> usize {
		self.points.len()
	}

	pub fn get_id(&self) -> i32 {
		self.id
	}

	pub fn translate_coord_scale(surface: &Rectangle, point: &Point, x: &dyn Axis, y: &dyn Axis) -> Point {
		let scale_x = x.get_scale_value(surface.width);
		let scale_y = y.get_scale_value(surface.height);

		let x = ((point.x() - x.get_range().min()) * scale_x) + surface.x;
		let y = surface.y - ((point.y() - y.get_range().min()) * scale_y);

		Point::new(x, y)
	}

	pub fn draw_text(&self, context: &Context, text: &String) {
		context.select_font_face("Sans", FontSlant::Normal, FontWeight::Normal);
		context.set_font_size(20.0);

		context.show_text(text);
	}
}
