use cairo::{Context, Rectangle};
use super::series::{AbstractSeries, Point, Series};
use super::axis::Axis;

pub struct SplineSeries {
	series: AbstractSeries,
}

impl SplineSeries {
	pub fn new(id: i32, name: String) -> Self {
		Self {
			series: AbstractSeries::new(id, name),
		}
	}
}

impl Series for SplineSeries {
	fn add_point(&mut self, point: Point) {
		self.series.add_point(point);
	}

	fn nb_points(&self) -> usize {
		self.series.nb_points()
	}

	fn get_id(&self) -> i32 {
		self.series.get_id()
	}

	fn draw_legend(&self, context: &Context, surface: &Rectangle, point: Point) {
		let x_end = point.x() + 20.;
		context.move_to(point.x(), point.y());
		context.line_to(x_end, point.y());

		context.move_to(x_end + 20., point.y());
		self.series.draw_text(context, &self.series.name);
		context.stroke();
	}

	fn draw(&self, context: &Context, surface: &Rectangle, x: &dyn Axis, y: &dyn Axis) {
		let mut cnt = 1;

		loop {
			if cnt + 1 >= self.nb_points() {
				break;
			}

			let previous = AbstractSeries::translate_coord_scale(surface, &self.series.points[cnt - 1], x, y);
			let current = AbstractSeries::translate_coord_scale(surface, &self.series.points[cnt], x, y);
			let next = AbstractSeries::translate_coord_scale(surface, &self.series.points[cnt + 1], x, y);

			context.move_to(previous.x(), previous.y());

			context.curve_to(previous.x(),
					 previous.y(),
					 current.x(),
					 current.y(),
					 next.x(),
					 next.y(),
					);

			cnt += 2;
		}

		context.stroke();
	}
}
