use cairo::{Context, Rectangle};

use super::axis::{AbstractAxis, Axis, AxisType, AxisOrientation, AxisPosition, Range};

pub struct VerticalAxis {
	axis: AbstractAxis,
}

impl VerticalAxis {
	pub fn new(description: String, axis_type: AxisType, position: AxisPosition, range: Range) -> Self {
		Self {
			axis: AbstractAxis::new(description, axis_type, AxisOrientation::VerticalAxis, position, range),
		}
	}

	pub fn draw_marker(&self, context: &Context, surface: &Rectangle) {
		let scale = self.get_scale_value(surface.height);

		for marker in &self.axis.markers {
			let rel_val = self.axis.get_relative_value(*marker);
			let val_str = self.axis.marker_text(*marker);
			let extends = context.text_extents(&val_str);

			context.move_to(surface.x - extends.width - 10., surface.y - rel_val * scale + extends.height / 2.);
			self.axis.draw_text(context, &val_str);

			context.move_to(surface.x, surface.y - rel_val * scale);
			context.line_to(surface.x + surface.width, surface.y - rel_val * scale);
		}
	}
}

impl Axis for VerticalAxis {
	fn add_marker(&mut self, marker: f64) {
		self.axis.add_marker(marker);
	}

	fn set_unit(&mut self, unit: String) {
		self.axis.set_unit(unit);
	}

	fn set_step(&mut self, step: f64) {
		self.axis.set_step(step);
	}

	fn set_orientation(&mut self, orientation: AxisOrientation) {
		self.axis.set_orientation(orientation);
	}

	fn set_range(&mut self, range: Range) {
		self.axis.set_range(range);
	}

	fn get_range(&self) -> Range {
		self.axis.get_range()
	}

	fn set_description(&mut self, description: String) {
		self.axis.set_description(description);
	}

	fn get_legend(&self) -> String {
		self.axis.get_legend()
	}

	fn get_scale_value(&self, range: f64) -> f64 {
		self.axis.get_scale_value(range)
	}

	fn draw(&self, context: &Context, surface: &Rectangle) {
		let legend = self.get_legend();
		let extends = context.text_extents(&legend);

		context.move_to(surface.x, surface.y);
		context.line_to(surface.x, 0.0);

		context.save();

		context.move_to(surface.x / 2., surface.y - surface.height / 2. + extends.width / 4.);
		context.rotate(-::std::f64::consts::FRAC_PI_2);
		self.axis.draw_text(context, &legend);
		context.restore();

		self.draw_marker(context, surface);
		context.stroke();
	}
}
