#[macro_use]
extern crate diesel;

use diesel::prelude::*;

mod drivers;
mod measures;
mod database;
mod gui;

use crate::database::schema::measures::columns::time;

use chrono::NaiveDate;

use crate::measures::MeasuresType::*;
use crate::measures::ReadingType;
use crate::measures::InsulinType;
use crate::measures::Measurement;

pub fn fake_read() -> Vec<Measurement> {
	let date = NaiveDate::from_ymd(2020, 02, 17).and_hms(00, 35, 0);
	let mut ret = Vec::new();

	ret.push(Measurement::new(date, Glucose(ReadingType::Automatic, 142)));
	ret.push(Measurement::new(date, Glucose(ReadingType::Manual, 75)));
	ret.push(Measurement::new(date, Glucose(ReadingType::Blood, 160)));
	ret.push(Measurement::new(date, Insulin(InsulinType::Rapid, 5.0)));
	ret.push(Measurement::new(date, Insulin(InsulinType::Long, 6.0)));

	ret
}

fn main() {
	let db_connection = database::establish_connection();

	match drivers::driver::find_hid_devices() {
		Some(mut gluco) => {
			match gluco.open() {
				Ok(_i) => {
					println!("Device openned!");
					// gluco.info();

					// println!("Patient name: {:?}", gluco.get_patient_name());
					// println!("Patient id: {:?}", gluco.get_patient_id());
					// println!("Glucometer version: {:?}", gluco.get_version());
					// println!("Glucometer serial number: {:?}", gluco.get_serial_number());
					// println!("Glucometer current date: {:?}", gluco.get_date());
					// println!("Number of records: {:?}", gluco.get_nb_records());

					let results = gluco.read().unwrap();
					for m in results.iter() {
						m.display();
						diesel::insert_into(database::schema::measures::table)
							.values(database::models::NewMeasure::from(m))
							.execute(&db_connection)
							.expect("Error saving new order");
					}
				},
				Err(err) => println!("Error to open glucometer: {}.", err),
			}
		},
		None => {
			println!("No glucometers found.");
			if false {
				let results = fake_read();
				for m in results.iter() {
					m.display();
					diesel::insert_into(database::schema::measures::table)
						.values(database::models::NewMeasure::from(m))
						.execute(&db_connection)
						.expect("Error saving new order");
				}
			} else {
				let results = database::schema::measures::table
					.filter(time.gt(NaiveDate::from_ymd(2020, 1, 27).and_hms(0, 0, 0)))
					.filter(time.lt(NaiveDate::from_ymd(2020, 1, 27).and_hms(23, 59, 59)))
					.load::<database::models::Measure>(&db_connection);
				for m in results {
					for measures in m {
						measures.to_measurement().display();
					}
				}

				gui::start_gui();
			}
		},
	};
}
