extern crate chrono;

pub enum InsulinType {
	Rapid,
	Long,
}

pub enum ReadingType {
	Automatic,
	Manual,
	Blood,
	BloodTest,
}

pub enum ConcentrationUnit {
	GPerLiter,
	MgPerDeciliter,
	MmolPerLiter,
}

pub enum MeasuresType {
	Glucose(ReadingType, u32),
	Ketone(u32),
	Insulin(InsulinType, f32),
	Food(u32),
	Sport(u32),
	Medications(bool),
}

pub struct Measurement {
	pub measure_type: MeasuresType,
	pub time: chrono::NaiveDateTime,
}

impl Measurement {
	pub fn new(time: chrono::NaiveDateTime, measure: MeasuresType) -> Self {
		Self {
			measure_type: measure,
			time: time,
		}
	}

	pub fn display(&self) {
		println!("{}", self.time);

		match &self.measure_type {
			MeasuresType::Glucose(t, val) => match t {
				ReadingType::Automatic => println!("Glucose from automatic: {} mg/dl", val),
				ReadingType::Manual => println!("Glucose from manual: {} mg/dl", val),
				ReadingType::Blood => println!("Glucose from blood: {} mg/dl", val),
				_ => println!("Glucose from blood test: {} mg/dl", val),
			},
			MeasuresType::Ketone(val) => println!("Ketone: {} mg/dl", val),
			MeasuresType::Insulin(t, val) => match t {
				InsulinType::Rapid => println!("Insulin rapid: {} units", val),
				_ => println!("Insulin long: {} units", val),
			},
			MeasuresType::Food(val) => println!("Food: {} g", val),
			MeasuresType::Sport(val) => println!("Sport: {}", val),
			MeasuresType::Medications(val) => println!("Medications: {}", val),
		}
	}
}
